import {
  LitElement,
  html,
  css,
  svg,
  property,
  TemplateResult
} from "lit-element";

import { mdiAsterisk } from "@mdi/js";

const SIZE = 35;
const DIST = 20;

interface GraphNode extends LitElement{
  render_svg(): TemplateResult;
  width: number;
  height: number;
}

class ScriptGraphNode extends LitElement {

  @property() icon = mdiAsterisk;

  get width() {
    return SIZE;
  }
  get height() {
    return SIZE + DIST;
  }

  render_svg() {
    return svg`
      <circle
        cx="0"
        cy="${this.width/2}"
        r="${SIZE/2}"
      />
      <g style="pointer-events: none" transform="translate(-12, ${this.width/2-12})">
      </g>
    `
  }

}

class ScriptGraph3 extends LitElement {

  @property() content = [];
  @property() _width = 0;
  @property() _height = 0;

  connectedCallback() {
    super.connectedCallback();
    console.log(this.querySelectorAll('*'))

  }

  async updateChildren() {
    this.content = [];
    let y = 0;
    let w = 0;
    for (const e of this.querySelectorAll('*') as NodeListOf<GraphNode>) {
      this.content.push({
        svg: e.render_svg(),
        offset_y: y,
      });
      y += e.height;
      w = Math.max(w, e.width);
    }
    this._width = w;
    this._height = y;
    this.requestUpdate();
  }

  childrenChangedCallback() {
    console.log("Children changed");
  }

  render() {


    let y = 0;
    let nodes = [];
    for (const e of this.content) {

    }

    return html`
    <svg width=500 height=500>
      ${this.content.map(e =>
        svg`
          <g transform="translate(${this._width/2} ${e.offset_y})">
          ${e.svg}
          </g>
        `)}
    </svg>
    <slot
    @slotchange=${this.updateChildren}
    ></slot>
    <style>
      slot {
        display: none;
      }
      </style>
    `
  }

}
customElements.define("script-graph3", ScriptGraph3);
customElements.define("script-graph-node", ScriptGraphNode);
